package main

import (
	"encoding/json"
	"fmt"
	"net/http"
	"strings"

	"github.com/gorilla/mux"
)

// TagRequest represents the request body for tag operations
type TagRequest struct {
	Name        string `json:"name"`
	Description string `json:"description"`
	Color       string `json:"color"`
}

// GetTagsHandler returns all tags
func GetTagsHandler(w http.ResponseWriter, r *http.Request) {
	log := GetLogger()
	loadItems := r.URL.Query().Get("include_items") == "true"

	var tags []Tag
	query := db.Model(&Tag{})

	// Only preload items if specifically requested
	if loadItems {
		query = query.Preload("Items")
	}

	if err := query.Find(&tags).Error; err != nil {
		log.Error("Failed to fetch tags: %v", err)
		http.Error(w, "Failed to fetch tags", http.StatusInternalServerError)
		return
	}
	json.NewEncoder(w).Encode(tags)
}

// CreateTagHandler creates a new tag
func CreateTagHandler(w http.ResponseWriter, r *http.Request) {
	log := GetLogger()
	var req TagRequest
	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
		log.Error("Failed to decode tag creation request: %v", err)
		http.Error(w, "Invalid request body", http.StatusBadRequest)
		return
	}

	// Validate tag name
	req.Name = strings.TrimSpace(req.Name)
	if req.Name == "" {
		http.Error(w, "Tag name is required", http.StatusBadRequest)
		return
	}

	// Check for duplicate tag name
	var existingTag Tag
	if err := db.Where("name = ?", req.Name).First(&existingTag).Error; err == nil {
		http.Error(w, "Tag name already exists", http.StatusConflict)
		return
	}

	tag := Tag{
		Name:        req.Name,
		Description: req.Description,
		Color:       req.Color,
	}

	if err := db.Create(&tag).Error; err != nil {
		log.Error("Failed to create tag: %v", err)
		http.Error(w, "Failed to create tag", http.StatusInternalServerError)
		return
	}

	log.DatabaseAction("create", fmt.Sprintf("Created tag: %s", tag.Name))
	json.NewEncoder(w).Encode(tag)
}

// UpdateTagHandler updates an existing tag
func UpdateTagHandler(w http.ResponseWriter, r *http.Request) {
	log := GetLogger()
	vars := mux.Vars(r)
	id := vars["id"]

	var tag Tag
	if err := db.First(&tag, id).Error; err != nil {
		http.Error(w, "Tag not found", http.StatusNotFound)
		return
	}

	var req TagRequest
	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
		http.Error(w, "Invalid request body", http.StatusBadRequest)
		return
	}

	// Check for duplicate name if name is being changed
	if req.Name != tag.Name {
		var existingTag Tag
		if err := db.Where("name = ? AND id != ?", req.Name, id).First(&existingTag).Error; err == nil {
			http.Error(w, "Tag name already exists", http.StatusConflict)
			return
		}
	}

	tag.Name = req.Name
	tag.Description = req.Description
	tag.Color = req.Color

	if err := db.Save(&tag).Error; err != nil {
		log.Error("Failed to update tag: %v", err)
		http.Error(w, "Failed to update tag", http.StatusInternalServerError)
		return
	}

	log.DatabaseAction("update", fmt.Sprintf("Updated tag: %s", tag.Name))
	json.NewEncoder(w).Encode(tag)
}

// DeleteTagHandler deletes a tag
func DeleteTagHandler(w http.ResponseWriter, r *http.Request) {
	log := GetLogger()
	vars := mux.Vars(r)
	id := vars["id"]

	// Begin transaction
	tx := db.Begin()

	// Remove tag associations
	if err := tx.Table("item_tags").Where("tag_id = ?", id).Delete(ItemTag{}).Error; err != nil {
		tx.Rollback()
		log.Error("Failed to remove tag associations: %v", err)
		http.Error(w, "Failed to delete tag", http.StatusInternalServerError)
		return
	}

	// Delete the tag
	if err := tx.Delete(&Tag{}, id).Error; err != nil {
		tx.Rollback()
		log.Error("Failed to delete tag: %v", err)
		http.Error(w, "Failed to delete tag", http.StatusInternalServerError)
		return
	}

	// Commit transaction
	if err := tx.Commit().Error; err != nil {
		log.Error("Failed to commit tag deletion: %v", err)
		http.Error(w, "Failed to delete tag", http.StatusInternalServerError)
		return
	}

	log.DatabaseAction("delete", fmt.Sprintf("Deleted tag ID: %s", id))
	w.WriteHeader(http.StatusNoContent)
}

// AddItemTagsHandler adds tags to an item
// AddItemTagsHandler adds tags to an item
func AddItemTagsHandler(w http.ResponseWriter, r *http.Request) {
	log := GetLogger()
	vars := mux.Vars(r)
	itemID := vars["id"]

	log.Info("Attempting to add tags to item %s", itemID)

	var item Item
	if err := db.First(&item, itemID).Error; err != nil {
		log.Error("Failed to find item %s: %v", itemID, err)
		http.Error(w, "Item not found", http.StatusNotFound)
		return
	}

	var tagIDs []uint
	if err := json.NewDecoder(r.Body).Decode(&tagIDs); err != nil {
		log.Error("Failed to decode tag IDs from request: %v", err)
		http.Error(w, "Invalid request body - expecting array of tag IDs", http.StatusBadRequest)
		return
	}

	log.Info("Received request to add tags %v to item %s", tagIDs, itemID)

	// Begin transaction
	tx := db.Begin()
	defer func() {
		if r := recover(); r != nil {
			tx.Rollback()
		}
	}()

	// First, fetch all the tags we want to add
	var tags []Tag
	if err := tx.Where("id IN (?)", tagIDs).Find(&tags).Error; err != nil {
		tx.Rollback()
		log.Error("Failed to fetch tags: %v", err)
		http.Error(w, "Failed to fetch tags", http.StatusInternalServerError)
		return
	}

	// Verify we found all requested tags
	if len(tags) != len(tagIDs) {
		tx.Rollback()
		log.Error("Not all tags were found. Requested: %d, Found: %d", len(tagIDs), len(tags))
		http.Error(w, "One or more tags not found", http.StatusBadRequest)
		return
	}

	// Direct SQL approach to insert associations
	for _, tag := range tags {
		// Check if association already exists using count
		var count int64
		err := tx.Table("item_tags").
			Where("item_id = ? AND tag_id = ?", item.ID, tag.ID).
			Count(&count).Error

		if err != nil {
			tx.Rollback()
			log.Error("Failed to check existing association: %v", err)
			http.Error(w, "Failed to check existing association", http.StatusInternalServerError)
			return
		}

		if count == 0 {
			// Insert new association
			err = tx.Exec("INSERT INTO item_tags (item_id, tag_id) VALUES (?, ?)",
				item.ID, tag.ID).Error
			if err != nil {
				tx.Rollback()
				log.Error("Failed to insert tag association: %v", err)
				http.Error(w, "Failed to add tag association", http.StatusInternalServerError)
				return
			}
			log.Info("Successfully added tag %d to item %s", tag.ID, itemID)
		} else {
			log.Info("Tag %d already associated with item %s", tag.ID, itemID)
		}
	}

	// Commit transaction
	if err := tx.Commit().Error; err != nil {
		log.Error("Failed to commit tag additions: %v", err)
		http.Error(w, "Failed to add tags", http.StatusInternalServerError)
		return
	}

	// Fetch updated item with tags
	var updatedItem Item
	if err := db.Preload("Tags").First(&updatedItem, itemID).Error; err != nil {
		log.Error("Failed to fetch updated item: %v", err)
		http.Error(w, "Failed to fetch updated item", http.StatusInternalServerError)
		return
	}

	log.DatabaseAction("update", fmt.Sprintf("Added tags %v to item %s", tagIDs, itemID))
	json.NewEncoder(w).Encode(updatedItem)
}

// RemoveItemTagHandler removes a tag from an item
func RemoveItemTagHandler(w http.ResponseWriter, r *http.Request) {
	log := GetLogger()
	vars := mux.Vars(r)
	itemID := vars["id"]
	tagID := vars["tagId"]

	log.Info("Attempting to remove tag %s from item %s", tagID, itemID)

	// Begin transaction
	tx := db.Begin()
	defer func() {
		if r := recover(); r != nil {
			tx.Rollback()
		}
	}()

	// Delete the association directly
	if err := tx.Exec("DELETE FROM item_tags WHERE item_id = ? AND tag_id = ?",
		itemID, tagID).Error; err != nil {
		tx.Rollback()
		log.Error("Failed to remove tag %s from item %s: %v", tagID, itemID, err)
		http.Error(w, "Failed to remove tag", http.StatusInternalServerError)
		return
	}

	// Commit transaction
	if err := tx.Commit().Error; err != nil {
		log.Error("Failed to commit tag removal: %v", err)
		http.Error(w, "Failed to remove tag", http.StatusInternalServerError)
		return
	}

	// Fetch updated item with remaining tags
	var updatedItem Item
	if err := db.Preload("Tags").First(&updatedItem, itemID).Error; err != nil {
		log.Error("Failed to fetch updated item: %v", err)
		http.Error(w, "Failed to fetch updated item", http.StatusInternalServerError)
		return
	}

	log.DatabaseAction("update", fmt.Sprintf("Removed tag %s from item %s", tagID, itemID))
	json.NewEncoder(w).Encode(updatedItem)
}

// GetItemsByTagHandler returns all items with a specific tag
func GetItemsByTagHandler(w http.ResponseWriter, r *http.Request) {
	log := GetLogger()
	vars := mux.Vars(r)
	tagID := vars["id"]

	var items []Item
	if err := db.Joins("JOIN item_tags ON items.id = item_tags.item_id").
		Where("item_tags.tag_id = ?", tagID).
		Preload("Tags"). // Always preload tags for items
		Find(&items).Error; err != nil {
		log.Error("Failed to fetch items by tag: %v", err)
		http.Error(w, "Failed to fetch items", http.StatusInternalServerError)
		return
	}

	json.NewEncoder(w).Encode(items)
}
